# -*- coding: UTF-8 -*-
import os
import sys
import platform
import subprocess
import CommonFileAPI
import pathlib
from typing import Dict

# config
my_project_name = 'PackProjectTest'
project_dir = 'D:/UEProjects/PackProjectTest'
engine_dir ='D:/Trunk/trunk/UnrealEngine-5.4/Engine'
patch_name = 'TestPatch'

cwd = os.path.dirname(__file__)
cwd.replace('\\','/')
sys.path.append(os.path.dirname(cwd))
sys.path.append(cwd)
host_name = platform.system().lower()
CookFlavor = 'ASTC'
bExecuteCmd = True

class MakeGamePakPatch:
    def __init__(self, InParamList={}):
        self.ParamList = {
            'TargetPlatform': 'Android_ASTC',
            'EngineDir': 'D:/Trunk/trunk/UnrealEngine-5.4/Engine',
            'ProjectDir': 'D:/UEProjects/PackProjectTest',
            'SvnNum': '',
            'Version': '1'
        }

        if host_name == 'darwin':
            self.ParamList = {
                'TargetPlatform': 'IOS',
                'EngineDir': '/Users/admin/Trunk/trunk/UnrealEngine-5.4/Engine',
                'ProjectDir': '/Users/admin/UEProjects/PackProjectTest',
                'SvnNum': '',
                'Version': '1'
            }

        self.ParamList['EngineDir'] = engine_dir
        self.ParamList['ProjectDir'] = project_dir

        self.ParseParam(InParamList)
        self.ChangedAssetList = []
        self.ChangedFileList = []

    def ParseParam(self, InParamList={}):
        print('try parse env params:' + str(InParamList))
        for (k, v) in InParamList.items():
            self.ParamList[k] = v
        for ele in self.ParamList.items():
            print('read param: ' + ele[0] + ' ' + ele[1])
        self.RefreshPath()

    def ReadUserDefinedFileList(self):
        DescFilePath = cwd + '/LocalPatchDesc.txt'
        fd = open(DescFilePath, 'r')
        if fd == None:
            print('ReadUserDefinedFileList!')
            raise Exception('LocalPatchDesc', DescFilePath)

        FileList = fd.readlines()
        for f in FileList:
            f = f.replace('\\', '/')
            f = f.replace("\n", "")
            f = f.replace("\r\n", "")
            # if f.startswith('Content/') or f.startswith('Config/'):
            #     if f.endswith('.uasset') or f.endswith('.umap'):
            #         self.ChangedAssetList.append(f)
            #     else:
            #         self.ChangedFileList.append(f)
            if f.endswith('.uasset') or f.endswith('.umap'):
                self.ChangedAssetList.append(f)
            else:
                self.ChangedFileList.append(f)
        fd.close()

    def RefreshPath(self):
        self.TargetPlatform = self.ParamList['TargetPlatform']
        self.EngineDir = self.ParamList['EngineDir']
        self.ProjectDir = self.ParamList['ProjectDir']

        self.FullPlatform = self.TargetPlatform
        self.bIsAndroid = self.FullPlatform.lower().find('android') > -1
        self.bIsIOS = self.FullPlatform.lower().find('ios') > -1
        if self.bIsAndroid and self.FullPlatform.lower().find(CookFlavor.lower()) < 0:
            self.FullPlatform = self.FullPlatform + '_' + CookFlavor

        self.EditorCmd = '{}/Binaries/Win64/UnrealEditor-Cmd.exe'.format(self.EngineDir)
        if CommonFileAPI.FileExist(self.EditorCmd) == False:
            self.EditorCmd = '{}/Binaries/Win64/UnrealEditor-Win64-DebugGame-Cmd.exe'.format(self.EngineDir)
        if host_name == 'darwin':
            self.UATApth = self.EngineDir + '/Binaries/Mac/UnrealEditor-Cmd'

    def PublishLocal(self, InVersion: str = ''):
        if InVersion == '':
            InVersion = self.ParamList['Version']
        self.ParseParam()

        # ---- Cook ----
        CookCmd = '{} "{}.uproject" -run=Cook -TargetPlatform={} -unattended -unversioned -DisableUnsolicitedPackages -cooksinglepackage'.format(self.EditorCmd, self.ProjectDir+'/'+my_project_name, self.TargetPlatform)
        
        CookList = []
        for f in self.ChangedAssetList:
            t = f
            if(isinstance(t, str)):
                ## 存在plugins
                if t.count("Plugins")>0:
                    split_value = t.split('/')
                    need_append = []
                    bAdd = False
                    for i in range(len(split_value)):
                        if bAdd:
                            need_append.append(split_value[i])
                            continue
                        if split_value[i] == 'Content':
                            bAdd = True
                            if i-1>=0:
                                need_append.append(split_value[i-1])
                    target = '/'+'/'.join(need_append)
                    target = target.replace('.umap', '')
                    target = target.replace('.uasset', '')
                    CookList.append(target)
                else:
                    split_value = t.split(my_project_name)
                    part2 = split_value[-1]
                    part2 = part2.replace('Content/', 'Game/')
                    part2 = part2.replace('.umap', '')
                    part2 = part2.replace('.uasset', '')
                    CookList.append(part2)
        CookCmd = CookCmd + ' -Map=' + '+'.join(CookList)

        CmdResult = 0
        if len(CookList) > 0:
            if bExecuteCmd:
                CmdResult = os.system(CookCmd)
        
        if(CmdResult != 0):
            print('failed to run cook and stage cmd!')
            raise Exception('Stage', self.FullPlatform, CmdResult)

        # ---- EndCook ---

        # 收集uasset和非asset资产
        # 存放非uasset
        PakList = []

        ## TODO 打二进制资产可能存在路径问题 
        RawFileFormats = ['.lua','.py', '.ini', '.mp4', '.wem', '.png', '.json', '.txt']
        for f in self.ChangedFileList:
            if RawFileFormats.count(pathlib.Path(f).suffix) > 0:
                FullPath = self.ProjectDir + '/' + f
                RelativePath = '../../../{}/{}'.format(my_project_name, f)
                PakList.append('"{}" "{}"\n'.format(FullPath, RelativePath))

        # 存放uasset资产
        IoStoreList = []
        for f in self.ChangedAssetList:
            if isinstance(f,str):
                if f.count('Plugins')>0:
                    split_value = f.split(my_project_name)
                    part2 = split_value[-1].replace('\\','/')
                    FullPath = '{}/Saved/Cooked/{}/{}'.format(self.ProjectDir, self.TargetPlatform, my_project_name + part2)
                    RelativePath = '../../../{}'.format(my_project_name + part2)
                    IoStoreList.append('"{}" "{}"\n'.format(FullPath, RelativePath))
                else:
                    split_value = f.split(my_project_name)
                    part2 = split_value[-1].replace('\\','/')
                    FullPath = '{}/Saved/Cooked/{}/{}'.format(self.ProjectDir, self.TargetPlatform, my_project_name + part2)
                    RelativePath = '../../../{}'.format(my_project_name + part2)
                    IoStoreList.append('"{}" "{}"\n'.format(FullPath, RelativePath))
            # FullPath_Uexp = self.ProjectDir + '/Saved/Cooked/' \
            #     + self.TargetPlatform + '/'+my_project_name+'/' + f.replace('.uasset', '.uexp')
            # RelativePath_Uexp = '../../../'+my_project_name+'/' + f.replace('.uasset', '.uexp')
            # if CommonFileAPI.FileExist(FullPath_Uexp):
            #     RespText.append( '\"{}\" \"{}\"\n'.format(FullPath_Uexp, RelativePath_Uexp) )

        temp_dir = cwd+"/temp/"
        temp_dir = temp_dir.replace("\\","/")
        if CommonFileAPI.DirectoryExist(temp_dir):
            CommonFileAPI.DeleteDirectory(temp_dir)
        os.mkdir(temp_dir)


        pakListPath = temp_dir+"PakList.txt"
        fd = open(pakListPath, 'w',encoding="utf-8")
        fd.writelines(PakList)
        fd.close()

        ioStoreListPath = temp_dir+"IOStore.txt"
        fd = open(ioStoreListPath, 'w',encoding="utf-8")
        fd.writelines(IoStoreList)
        fd.close()
        pakCommandsPath = temp_dir+"PakCommands.txt"
        fd = open(pakCommandsPath, 'w',encoding="utf-8")
        pakName = '{}_{}_{}_p'.format(patch_name, self.TargetPlatform, InVersion)
        targetPakPath = '{}/Saved/StagedBuilds/{}/{}/Content/Paks/{}.pak'.format(self.ProjectDir, self.TargetPlatform, my_project_name, pakName)
        fd.write('"{}" -create="{}"'.format(targetPakPath.replace("\\","/"), pakListPath))
        fd.close()

        UnrealPak = self.EngineDir + '/Binaries/Win64/UnrealPak.exe'
        if host_name == 'darwin':
            UnrealPak = self.EngineDir + '/Binaries/Mac/UnrealPak.exe'
        CreatePakCmd = '{} "{}.uproject"  -patchpaddingalign=0 -compressionformats=Oodle -compresslevel=4 -compressmethod=Mermaid\
              -platform=Android  -CreateMultiple="{}"'.format(UnrealPak, self.ProjectDir+'/'+my_project_name, pakCommandsPath)
        if bExecuteCmd:
            PakResult = os.system(CreatePakCmd)

        ioStoreCommandsPath = temp_dir+"IoStoreCommands.txt"
        fd = open(ioStoreCommandsPath, 'w', encoding="utf-8")
        targetUtocPath = '{}/Saved/StagedBuilds/{}/{}/Content/Paks/{}.utoc'.format(self.ProjectDir, self.TargetPlatform, my_project_name, pakName)
        fd.write('-Output="{}" -ContainerName={} -ResponseFile="{}"'.format(targetUtocPath, pakName, ioStoreListPath))
        fd.close()

        global_utoc_path = '{}/Saved/StagedBuilds/{}/{}/Content/Paks/global.utoc'.format(self.ProjectDir, self.TargetPlatform, my_project_name)
        cook_directory = '{}/Saved/Cooked/{}'.format(self.ProjectDir, self.TargetPlatform)
        PackageStoreManifest = '{}/Saved/Cooked/{}/{}/Metadata/packagestore.manifest'.format(self.ProjectDir, self.TargetPlatform, my_project_name)
        ScriptObjects = '{}/Saved/Cooked/{}/{}/Metadata/scriptobjects.bin'.format(self.ProjectDir, self.TargetPlatform, my_project_name)
        IOStoreCmd = '{} \
            -CreateGlobalContainer="{}" \
                -CookedDirectory="{}" \
                    -PackageStoreManifest="{}" \
                        -Commands="{}" \
                            -ScriptObjects="{}" \
                                -patchpaddingalign=0 -compressionformats=Oodle -compresslevel=4 -compressmethod=Mermaid \
                                    -compressionMinBytesSaved=1024 -compressionMinPercentSaved=5 -WriteBackMetadataToAssetRegistry=Disabled'\
                                        .format(UnrealPak, global_utoc_path, cook_directory, PackageStoreManifest, ioStoreCommandsPath, ScriptObjects)
        if bExecuteCmd:
            IoStoreResult = os.system(IOStoreCmd)

if __name__ == '__main__':
    Params: Dict[str, str] = {}
    for ele in sys.argv:
        arr = ele.split('=')
        if len(arr) == 2:
            Params[arr[0]] = arr[1]

    inst = MakeGamePakPatch(Params)
    inst.ReadUserDefinedFileList()
    inst.PublishLocal()
